/*
 *  AVRVM, a virtual machine which emulates an Atmel AVR CPU.
 *  Copyright (C) 2010  Joakim Soderlund
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "cpu.h"

typedef struct {
	unsigned short filter;
	unsigned short mask;
	int (*function)(ATMega16 *cpu);
} AVRVMInstruction;

extern AVRVMInstruction avrvm_instructions[];

#define AVRVM_LDI_FILTER   0xF000 // 1111 0000 0000 0000
#define AVRVM_LDI_MASK     0xE000 // 1110 KKKK dddd KKKK
int avrvm_ldi(ATMega16 *cpu);

#define AVRVM_AND_FILTER   0xFC00 // 1111 1100 0000 0000
#define AVRVM_AND_MASK     0x2000 // 0020 00rd dddd rrrr
int avrvm_and(ATMega16 *cpu);

#define AVRVM_RJMP_FILTER  0xF000 // 1111 0000 0000 0000
#define AVRVM_RJMP_MASK    0xC000 // 1100 kkkk kkkk kkkk
int avrvm_rjmp(ATMega16 *cpu);

#define AVRVM_SEZ_FILTER   0xFFFF // 1111 1111 1111 1111
#define AVRVM_SEZ_MASK     0x9418 // 1001 0100 0001 1000
int avrvm_sez(ATMega16 *cpu);

#define AVRVM_SEV_FILTER   0xFFFF // 1111 1111 1111 1111
#define AVRVM_SEV_MASK     0x9438 // 1001 0100 0011 1000
int avrvm_sev(ATMega16 *cpu);

#define AVRVM_OUT_FILTER   0xF800 // 1111 1000 0000 0000
#define AVRVM_OUT_MASK     0xB800 // 1011 1AAr rrrr AAAA
int avrvm_out(ATMega16 *cpu);

#define AVRVM_IN_FILTER    0xF800 // 1111 1000 0000 0000
#define AVRVM_IN_MASK      0xB000 // 1011 0AAr rrrr AAAA
int avrvm_in(ATMega16 *cpu);

#define AVRVM_PUSH_FILTER  0xFE0F // 1111 1110 0000 1111
#define AVRVM_PUSH_MASK    0x920F // 1001 001d dddd 1111
int avrvm_push(ATMega16 *cpu);

#define AVRVM_POP_FILTER   0xFE0F // 1111 1110 0000 1111
#define AVRVM_POP_MASK     0x900F // 1001 000d dddd 1111
int avrvm_pop(ATMega16 *cpu);

#define AVRVM_RCALL_FILTER 0xF000 // 1111 0000 0000 0000
#define AVRVM_RCALL_MASK   0xD000 // 1101 kkkk kkkk kkkk
int avrvm_rcall(ATMega16 *cpu);

#define AVRVM_RET_FILTER   0xFFFF // 1111 1111 1111 1111
#define AVRVM_RET_MASK     0x9508 // 1001 0101 0000 1000
int avrvm_ret(ATMega16 *cpu);

#define AVRVM_NOP_FILTER   0xFFFF // 1111 1111 1111 1111
#define AVRVM_NOP_MASK     0x0000 // 0000 0000 0000 0000
int avrvm_nop(ATMega16 *cpu);

#define AVRVM_ANDI_FILTER  0xF000 // 1111 0000 0000 0000
#define AVRVM_ANDI_MASK    0x7000 // 0111 KKKK dddd KKKK
int avrvm_andi(ATMega16 *cpu);

#define AVRVM_OR_FILTER    0xFC00 // 1111 1100 0000 0000
#define AVRVM_OR_MASK      0x2800 // 0010 10rd dddd rrrr
int avrvm_or(ATMega16 *cpu);

#define AVRVM_ORI_FILTER   0xF000 // 1111 0000 0000 0000
#define AVRVM_ORI_MASK     0x6000 // 0110 KKKK dddd KKKK
int avrvm_ori(ATMega16 *cpu);

#define AVRVM_BREQ_FILTER  0xFC07 // 1111 1100 0000 0111
#define AVRVM_BREQ_MASK    0xF001 // 1111 00kk kkkk k001
int avrvm_breq(ATMega16 *cpu);

#define AVRVM_BRNE_FILTER  0xFC07 // 1111 1100 0000 0111
#define AVRVM_BRNE_MASK    0xF401 // 1111 01kk kkkk k001
int avrvm_brne(ATMega16 *cpu);

#define AVRVM_CP_FILTER    0xFC00 // 1111 1100 0000 0000
#define AVRVM_CP_MASK      0x1400 // 0001 01rd dddd rrrr
int avrvm_cp(ATMega16 *cpu);

